@@ -197,7 +197,8 @@ const Config::ConfigEntry Config::entries[MAX_CONFIG_KEY] =
 	{TYPE_BOOLEAN,		"IPv6V6Only",				(ConfigValue) false},
 	{TYPE_BOOLEAN,		"WireCompression",			(ConfigValue) false},
 	{TYPE_BOOLEAN,		"AllowEncryptedSecurityDatabase", (ConfigValue) false},
-	{TYPE_BOOLEAN,		"ClearGTTAtRetaining",		(ConfigValue) true}
+	{TYPE_BOOLEAN,		"ClearGTTAtRetaining",		(ConfigValue) true},
+	{TYPE_BOOLEAN,		"LegacyDatabaseFileId",		(ConfigValue) true}
 };
 
 /******************************************************************************
@@ -451,9 +452,9 @@ int Config::getGuardianOption()
 	return (int) getDefaultConfig()->values[KEY_GUARDIAN_OPTION];
 }
 
-int Config::getCpuAffinityMask()
+FB_UINT64 Config::getCpuAffinityMask()
 {
-	return (int) getDefaultConfig()->values[KEY_CPU_AFFINITY_MASK];
+	return (FB_UINT64) getDefaultConfig()->values[KEY_CPU_AFFINITY_MASK];
 }
 
 int Config::getTcpRemoteBufferSize()
@@ -824,3 +825,8 @@ bool Config::getClearGTTAtRetaining() const
 {
 	return get<bool>(KEY_CLEAR_GTT_RETAINING);
 }
+
+bool Config::getLegacyDatabaseFileId()
+{
+	return (bool) getDefaultConfig()->values[KEY_LEGACY_DB_FILE_ID];
+}
