@@ -52,13 +52,13 @@ using namespace Firebird;
 
 
 // static method
-Jrd::Attachment* Jrd::Attachment::create(Database* dbb)
+Jrd::Attachment* Jrd::Attachment::create(Database* dbb, JProvider* provider)
 {
 	MemoryPool* const pool = dbb->createPool();
 
 	try
 	{
-		Attachment* const attachment = FB_NEW_POOL(*pool) Attachment(pool, dbb);
+		Attachment* const attachment = FB_NEW_POOL(*pool) Attachment(pool, dbb, provider);
 		pool->setStatsGroup(attachment->att_memory_stats);
 		return attachment;
 	}
@@ -171,10 +171,11 @@ void Jrd::Attachment::backupStateReadUnLock(thread_db* tdbb)
 }
 
 
-Jrd::Attachment::Attachment(MemoryPool* pool, Database* dbb)
+Jrd::Attachment::Attachment(MemoryPool* pool, Database* dbb, JProvider* provider)
 	: att_pool(pool),
 	  att_memory_stats(&dbb->dbb_memory_stats),
 	  att_database(dbb),
+	  att_user_ids(*pool),
 	  att_requests(*pool),
 	  att_lock_owner_id(Database::getLockOwnerId()),
 	  att_backup_state_counter(0),
@@ -204,7 +205,8 @@ Jrd::Attachment::Attachment(MemoryPool* pool, Database* dbb)
 	  att_dyn_req(*pool),
 	  att_charsets(*pool),
 	  att_charset_ids(*pool),
-	  att_pools(*pool)
+	  att_pools(*pool),
+	  att_provider(provider)
 {
 	att_internal.grow(irq_MAX);
 	att_dyn_req.grow(drq_MAX);
@@ -215,6 +217,20 @@ Jrd::Attachment::~Attachment()
 {
 	delete att_trace_manager;
 
+	for (Function** iter = att_functions.begin(); iter < att_functions.end(); ++iter)
+	{
+		Function* const function = *iter;
+		if (function)
+			delete function;
+	}
+
+	for (jrd_prc** iter = att_procedures.begin(); iter < att_procedures.end(); ++iter)
+	{
+		jrd_prc* const procedure = *iter;
+		if (procedure)
+			delete procedure;
+	}
+
 	while (att_pools.hasData())
 		deletePool(att_pools.pop());
 
@@ -765,3 +781,18 @@ JAttachment* Attachment::getInterface() throw()
 	return att_stable->getInterface();
 }
 
+UserId* Attachment::getUserId(const string& userName)
+{
+	// It's necessary to keep specified sql role of user
+	if (att_user && att_user->usr_user_name == userName)
+		return att_user;
+
+	UserId* result = NULL;
+	if (!att_user_ids.get(userName, result))
+	{
+		result = FB_NEW_POOL(*att_pool) UserId(*att_pool);
+		result->usr_user_name = userName;
+		att_user_ids.put(userName, result);
+	}
+	return result;
+}
