@@ -286,7 +286,7 @@ namespace Jrd {
 		  slowIO(0),
 		  crypt(false),
 		  process(false),
-		  down(false),
+		  flDown(false),
 		  run(false)
 	{
 		stateLock = FB_NEW_RPT(getPool(), 0)
@@ -308,7 +308,7 @@ namespace Jrd {
 
 	void CryptoManager::shutdown(thread_db* tdbb)
 	{
-		terminateCryptThread(tdbb);
+		terminateCryptThread(tdbb, false);
 
 		if (cryptPlugin)
 		{
@@ -365,7 +365,9 @@ namespace Jrd {
 		crypt = hdr->hdr_flags & Ods::hdr_encrypted;
 		process = hdr->hdr_flags & Ods::hdr_crypt_process;
 
-		if ((crypt || process) && !cryptPlugin)
+		// tdbb w/o attachment comes when database is shutting down in the end of detachDatabase()
+		// the only needed here page is header, i.e. we can live w/o cryptPlugin
+		if ((crypt || process) && (!cryptPlugin) && tdbb->getAttachment())
 		{
 			ClumpletWriter hc(ClumpletWriter::UnTagged, hdr->hdr_page_size);
 			hdr.getClumplets(hc);
@@ -388,7 +390,7 @@ namespace Jrd {
 				hash = valid;
 		}
 
-		if (flags & CRYPT_HDR_INIT)
+		if (cryptPlugin && (flags & CRYPT_HDR_INIT))
 			checkDigitalSignature(tdbb, hdr);
 	}
 
@@ -825,7 +827,7 @@ namespace Jrd {
 
 	void CryptoManager::terminateCryptThread(thread_db*, bool wait)
 	{
-		down = true;
+		flDown = true;
 		if (wait && cryptThreadId)
 		{
 			Thread::waitForCompletion(cryptThreadId);
@@ -836,7 +838,7 @@ namespace Jrd {
 	void CryptoManager::stopThreadUsing(thread_db* tdbb, Attachment* att)
 	{
 		if (att == cryptAtt)
-			terminateCryptThread(tdbb);
+			terminateCryptThread(tdbb, false);
 	}
 
 	void CryptoManager::startCryptThread(thread_db* tdbb)
@@ -864,10 +866,6 @@ namespace Jrd {
 		bool releasingLock = false;
 		try
 		{
-			// Cleanup resources
-			terminateCryptThread(tdbb);
-			down = false;
-
 			// Determine current page from the header
 			CchHdr hdr(tdbb, LCK_read);
 			process = hdr->hdr_flags & Ods::hdr_crypt_process ? true : false;
@@ -930,7 +928,7 @@ namespace Jrd {
 			UserId user;
 			user.usr_user_name = "Database Crypter";
 
-			Jrd::Attachment* const attachment = Jrd::Attachment::create(&dbb);
+			Jrd::Attachment* const attachment = Jrd::Attachment::create(&dbb, NULL);
 			RefPtr<SysStableAttachment> sAtt(FB_NEW SysStableAttachment(attachment));
 			attachment->setStable(sAtt);
 			attachment->att_filename = dbb.dbb_filename;
@@ -966,114 +964,114 @@ namespace Jrd {
 				writer.insertByte(isc_dpb_no_db_triggers, TRUE);
 
 				// Avoid races with release_attachment() in jrd.cpp
-				MutexEnsureUnlock releaseGuard(cryptAttMutex, FB_FUNCTION);
+				XThreadEnsureUnlock releaseGuard(dbb.dbb_thread_mutex, FB_FUNCTION);
 				releaseGuard.enter();
-				if (!down)
+				if (!down())
 				{
-				AutoPlugin<JProvider> jInstance(JProvider::getInstance());
-				jInstance->setDbCryptCallback(&status_vector, dbb.dbb_callback);
-				check(&status_vector);
+					AutoPlugin<JProvider> jInstance(JProvider::getInstance());
+					jInstance->setDbCryptCallback(&status_vector, dbb.dbb_callback);
+					check(&status_vector);
 
-				RefPtr<JAttachment> jAtt(REF_NO_INCR, jInstance->attachDatabase(&status_vector,
-					dbb.dbb_database_name.c_str(), writer.getBufferLength(), writer.getBuffer()));
-				check(&status_vector);
+					RefPtr<JAttachment> jAtt(REF_NO_INCR, jInstance->attachDatabase(&status_vector,
+						dbb.dbb_database_name.c_str(), writer.getBufferLength(), writer.getBuffer()));
+					check(&status_vector);
 
-				MutexLockGuard attGuard(*(jAtt->getStable()->getMutex()), FB_FUNCTION);
-				Attachment* att = jAtt->getHandle();
-				if (!att)
-					Arg::Gds(isc_att_shutdown).raise();
-				att->att_flags |= ATT_crypt_thread;
-				releaseGuard.leave();
+					MutexLockGuard attGuard(*(jAtt->getStable()->getMutex()), FB_FUNCTION);
+					Attachment* att = jAtt->getHandle();
+					if (!att)
+						Arg::Gds(isc_att_shutdown).raise();
+					att->att_flags |= ATT_from_thread;
+					releaseGuard.leave();
 
-				ThreadContextHolder tdbb(att->att_database, att, &status_vector);
-				tdbb->markAsSweeper();
+					ThreadContextHolder tdbb(att->att_database, att, &status_vector);
+					tdbb->markAsSweeper();
 
-				DatabaseContextHolder dbHolder(tdbb);
+					DatabaseContextHolder dbHolder(tdbb);
 
-				class UseCountHolder
-				{
-				public:
-					explicit UseCountHolder(Attachment* a)
-						: att(a)
+					class UseCountHolder
 					{
-						att->att_use_count++;
-					}
-					~UseCountHolder()
-					{
-						att->att_use_count--;
-					}
-				private:
-					Attachment* att;
-				};
-				UseCountHolder use_count(att);
+					public:
+						explicit UseCountHolder(Attachment* a)
+							: att(a)
+						{
+							att->att_use_count++;
+						}
+						~UseCountHolder()
+						{
+							att->att_use_count--;
+						}
+					private:
+						Attachment* att;
+					};
+					UseCountHolder use_count(att);
 
-				// get ready...
-				AutoSetRestore<Attachment*> attSet(&cryptAtt, att);
-				ULONG lastPage = getLastPage(tdbb);
+					// get ready...
+					AutoSetRestore<Attachment*> attSet(&cryptAtt, att);
+					ULONG lastPage = getLastPage(tdbb);
 
-				do
-				{
-					// Check is there some job to do
-					while (currentPage < lastPage)
+					do
 					{
+						// Check is there some job to do
+						while (currentPage < lastPage)
+						{
+							// forced terminate
+							if (down())
+							{
+								break;
+							}
+
+							// scheduling
+							if (--tdbb->tdbb_quantum < 0)
+								JRD_reschedule(tdbb, true);
+
+							// nbackup state check
+							BackupManager::StateReadGuard::lock(tdbb, 1);
+							int bak_state = tdbb->getDatabase()->dbb_backup_manager->getState();
+							BackupManager::StateReadGuard::unlock(tdbb);
+
+							if (bak_state != Ods::hdr_nbak_normal)
+							{
+								EngineCheckout checkout(tdbb, FB_FUNCTION);
+								Thread::sleep(10);
+								continue;
+							}
+
+							// writing page to disk will change it's crypt status in usual way
+							WIN window(DB_PAGE_SPACE, currentPage);
+							Ods::pag* page = CCH_FETCH(tdbb, &window, LCK_write, pag_undefined);
+							if (page && page->pag_type <= pag_max &&
+								(bool(page->pag_flags & Ods::crypted_page) != crypt) &&
+								Ods::pag_crypt_page[page->pag_type])
+							{
+								CCH_MARK_MUST_WRITE(tdbb, &window);
+							}
+							CCH_RELEASE_TAIL(tdbb, &window);
+
+							// sometimes save currentPage into DB header
+							++currentPage;
+							if ((currentPage & 0x3FF) == 0)
+							{
+								writeDbHeader(tdbb, currentPage);
+							}
+						}
+
 						// forced terminate
-						if (down)
+						if (down())
 						{
 							break;
 						}
 
-						// scheduling
-						if (--tdbb->tdbb_quantum < 0)
-							JRD_reschedule(tdbb, true);
+						// At this moment of time all pages with number < lastpage
+						// are guaranteed to change crypt state. Check for added pages.
+						lastPage = getLastPage(tdbb);
 
-						// nbackup state check
-						BackupManager::StateReadGuard::lock(tdbb, 1);
-						int bak_state = tdbb->getDatabase()->dbb_backup_manager->getState();
-						BackupManager::StateReadGuard::unlock(tdbb);
+					} while (currentPage < lastPage);
 
-						if (bak_state != Ods::hdr_nbak_normal)
-						{
-							EngineCheckout checkout(tdbb, FB_FUNCTION);
-							Thread::sleep(10);
-							continue;
-						}
-
-						// writing page to disk will change it's crypt status in usual way
-						WIN window(DB_PAGE_SPACE, currentPage);
-						Ods::pag* page = CCH_FETCH(tdbb, &window, LCK_write, pag_undefined);
-						if (page && page->pag_type <= pag_max &&
-							(bool(page->pag_flags & Ods::crypted_page) != crypt) &&
-							Ods::pag_crypt_page[page->pag_type])
-						{
-							CCH_MARK_MUST_WRITE(tdbb, &window);
-						}
-						CCH_RELEASE_TAIL(tdbb, &window);
-
-						// sometimes save currentPage into DB header
-						++currentPage;
-						if ((currentPage & 0x3FF) == 0)
-						{
-							writeDbHeader(tdbb, currentPage);
-						}
-					}
-
-					// forced terminate
-					if (down)
+					// Finalize crypt
+					if (!down())
 					{
-						break;
+						writeDbHeader(tdbb, 0);
 					}
-
-					// At this moment of time all pages with number < lastpage
-					// are guaranteed to change crypt state. Check for added pages.
-					lastPage = getLastPage(tdbb);
-
-				} while (currentPage < lastPage);
-
-				// Finalize crypt
-				if (!down)
-				{
-					writeDbHeader(tdbb, 0);
-				}
 				}
 
 				// Release exclusive lock on StartCryptThread
@@ -1348,6 +1346,11 @@ namespace Jrd {
 		return keyName.c_str();
 	}
 
+	bool CryptoManager::down() const
+	{
+		return flDown;
+	}
+
 	void CryptoManager::addClumplet(string& signature, ClumpletReader& block, UCHAR tag)
 	{
 		if (block.find(tag))
