@@ -67,7 +67,7 @@ JrdStatement::JrdStatement(thread_db* tdbb, MemoryPool* p, CompilerScratch* csb)
 		makeSubRoutines(tdbb, this, csb, csb->subProcedures);
 		makeSubRoutines(tdbb, this, csb, csb->subFunctions);
 
-		topNode = (csb->csb_node->kind == DmlNode::KIND_STATEMENT) ?
+		topNode = (csb->csb_node->getKind() == DmlNode::KIND_STATEMENT) ?
 			static_cast<StmtNode*>(csb->csb_node) : NULL;
 
 		accessList = csb->csb_access;
@@ -225,7 +225,7 @@ JrdStatement* JrdStatement::makeStatement(thread_db* tdbb, CompilerScratch* csb,
 
 			if (fieldInfo.validationExpr)
 			{
-				NodeCopier copier(csb, map);
+				NodeCopier copier(csb->csb_pool, csb, map);
 				fieldInfo.validationExpr = copier.copy(tdbb, fieldInfo.validationExpr);
 			}
 
@@ -233,7 +233,7 @@ JrdStatement* JrdStatement::makeStatement(thread_db* tdbb, CompilerScratch* csb,
 			DmlNode::doPass1(tdbb, csb, fieldInfo.validationExpr.getAddress());
 		}
 
-		if (csb->csb_node->kind == DmlNode::KIND_STATEMENT)
+		if (csb->csb_node->getKind() == DmlNode::KIND_STATEMENT)
 			StmtNode::doPass2(tdbb, csb, reinterpret_cast<StmtNode**>(&csb->csb_node), NULL);
 		else
 			ExprNode::doPass2(tdbb, csb, &csb->csb_node);
@@ -317,7 +317,7 @@ bool JrdStatement::isActive() const
 	return false;
 }
 
-jrd_req* JrdStatement::findRequest(thread_db* tdbb)
+jrd_req* JrdStatement::findRequest(thread_db* tdbb, bool unique)
 {
 	SET_TDBB(tdbb);
 	Attachment* const attachment = tdbb->getAttachment();
@@ -346,6 +346,9 @@ jrd_req* JrdStatement::findRequest(thread_db* tdbb)
 				break;
 			}
 
+			if (unique)
+				return NULL;
+
 			++count;
 		}
 		else if (!(next->req_flags & req_in_use) && !clone)
@@ -467,16 +470,17 @@ void JrdStatement::verifyAccess(thread_db* tdbb)
 			 ++access)
 		{
 			const SecurityClass* sec_class = SCL_get_class(tdbb, access->acc_security_name.c_str());
+			const SecurityClass* v_sec_class = SCL_get_class_for_view(tdbb, access->acc_view_id, access->acc_security_name.c_str());
 
 			if (routine->getName().package.isEmpty())
 			{
-				SCL_check_access(tdbb, sec_class, access->acc_view_id, aclType,
+				SCL_check_access(tdbb, sec_class, v_sec_class, aclType,
 					routine->getName().identifier, access->acc_mask, access->acc_type,
 					true, access->acc_name, access->acc_r_name);
 			}
 			else
 			{
-				SCL_check_access(tdbb, sec_class, access->acc_view_id,
+				SCL_check_access(tdbb, sec_class, v_sec_class,
 					id_package, routine->getName().package,
 					access->acc_mask, access->acc_type,
 					true, access->acc_name, access->acc_r_name);
@@ -495,6 +499,7 @@ void JrdStatement::verifyAccess(thread_db* tdbb)
 	for (const AccessItem* access = accessList.begin(); access != accessList.end(); ++access)
 	{
 		const SecurityClass* sec_class = SCL_get_class(tdbb, access->acc_security_name.c_str());
+		const SecurityClass* v_sec_class = SCL_get_class_for_view(tdbb, access->acc_view_id, access->acc_security_name.c_str());
 
 		MetaName objName;
 		SLONG objType = 0;
@@ -525,7 +530,7 @@ void JrdStatement::verifyAccess(thread_db* tdbb)
 			objName = transaction->tra_caller_name.name;
 		}
 
-		SCL_check_access(tdbb, sec_class, access->acc_view_id, objType, objName,
+		SCL_check_access(tdbb, sec_class, v_sec_class, objType, objName,
 			access->acc_mask, access->acc_type, true, access->acc_name, access->acc_r_name);
 	}
 }
@@ -644,8 +649,9 @@ void JrdStatement::verifyTriggerAccess(thread_db* tdbb, jrd_rel* ownerRelation,
 
 			// a direct access to an object from this trigger
 			const SecurityClass* sec_class = SCL_get_class(tdbb, access->acc_security_name.c_str());
-			SCL_check_access(tdbb, sec_class,
-				(access->acc_view_id) ? access->acc_view_id : (view ? view->rel_id : 0),
+			const SecurityClass* v_sec_class = SCL_get_class_for_view(tdbb,
+				access->acc_view_id ? access->acc_view_id : view ? view->rel_id : 0, access->acc_security_name.c_str());
+			SCL_check_access(tdbb, sec_class, v_sec_class,
 				id_trigger, t.statement->triggerName, access->acc_mask,
 				access->acc_type, true, access->acc_name, access->acc_r_name);
 		}
@@ -744,21 +750,6 @@ template <typename T> static void makeSubRoutines(thread_db* tdbb, JrdStatement*
 		subStatement->parentStatement = statement;
 		subRoutine->setStatement(subStatement);
 
-		switch (subRoutine->getObjectType())
-		{
-		case obj_procedure:
-			subStatement->procedure = static_cast<jrd_prc*>(subRoutine);
-			break;
-
-		case obj_udf:
-			subStatement->function = static_cast<Function*>(subRoutine);
-			break;
-
-		default:
-			fb_assert(false);
-			break;
-		}
-
 		// Move dependencies and permissions from the sub routine to the parent.
 
 		for (CompilerScratch::Dependency* dependency = subCsb->csb_dependencies.begin();
