@@ -60,7 +60,7 @@ namespace Jrd {
 
 // Check the index for being an expression one and
 // matching both the given stream and the given expression tree
-bool checkExpressionIndex(const index_desc* idx, ValueExprNode* node, StreamType stream)
+bool checkExpressionIndex(CompilerScratch* csb, const index_desc* idx, ValueExprNode* node, StreamType stream)
 {
 	fb_assert(idx);
 
@@ -68,7 +68,7 @@ bool checkExpressionIndex(const index_desc* idx, ValueExprNode* node, StreamType
 	{
 		// The desired expression can be hidden inside a derived expression node,
 		// so try to recover it (see CORE-4118).
-		while (!idx->idx_expression->sameAs(node, true))
+		while (!idx->idx_expression->sameAs(csb, node, true))
 		{
 			DerivedExprNode* const derivedExpr = node->as<DerivedExprNode>();
 			CastNode* const cast = node->as<CastNode>();
@@ -82,8 +82,8 @@ bool checkExpressionIndex(const index_desc* idx, ValueExprNode* node, StreamType
 		}
 
 		SortedStreamList exprStreams, nodeStreams;
-		idx->idx_expression->collectStreams(exprStreams);
-		node->collectStreams(nodeStreams);
+		idx->idx_expression->collectStreams(csb, exprStreams);
+		node->collectStreams(csb, nodeStreams);
 
 		if (exprStreams.getCount() == 1 && exprStreams[0] == 0 &&
 			nodeStreams.getCount() == 1 && nodeStreams[0] == stream)
@@ -720,10 +720,10 @@ void OptimizerRetrieval::analyzeNavigation(const InversionCandidateList& inversi
 					ValueExprNode* const node1 = cmpNode->arg1;
 					ValueExprNode* const node2 = cmpNode->arg2;
 
-					if (node1->sameAs(orgNode, false))
+					if (node1->sameAs(csb, orgNode, false))
 						nodes.add(node2);
 
-					if (node2->sameAs(orgNode, false))
+					if (node2->sameAs(csb, orgNode, false))
 						nodes.add(node1);
 				}
 			}
@@ -736,7 +736,7 @@ void OptimizerRetrieval::analyzeNavigation(const InversionCandidateList& inversi
 
 				if (idx->idx_flags & idx_expressn)
 				{
-					if (!checkExpressionIndex(idx, node, stream))
+					if (!checkExpressionIndex(csb, idx, node, stream))
 						continue;
 				}
 				else if (!(fieldNode = node->as<FieldNode>()) || fieldNode->fieldStream != stream)
@@ -1755,11 +1755,11 @@ bool OptimizerRetrieval::matchBoolean(IndexScratch* indexScratch, BoolExprNode*
 
 	    fb_assert(indexScratch->idx->idx_expression != NULL);
 
-		if (!checkExpressionIndex(indexScratch->idx, match, stream) ||
+		if (!checkExpressionIndex(csb, indexScratch->idx, match, stream) ||
 			(value && !value->computable(csb, stream, false)))
 		{
 			if ((!cmpNode || cmpNode->blrOp != blr_starting) && value &&
-				checkExpressionIndex(indexScratch->idx, value, stream) &&
+				checkExpressionIndex(csb, indexScratch->idx, value, stream) &&
 				match->computable(csb, stream, false))
 			{
 				ValueExprNode* temp = match;
@@ -1802,38 +1802,18 @@ bool OptimizerRetrieval::matchBoolean(IndexScratch* indexScratch, BoolExprNode*
 
 	bool excludeBound = cmpNode && (cmpNode->blrOp == blr_gtr || cmpNode->blrOp == blr_lss);
 
+	dsc matchDesc, valueDesc;
+
 	if (value)
 	{
-		dsc desc1, desc2;
-		match->getDesc(tdbb, csb, &desc1);
-		value->getDesc(tdbb, csb, &desc2);
+		match->getDesc(tdbb, csb, &matchDesc);
+		value->getDesc(tdbb, csb, &valueDesc);
 
-		if (!BTR_types_comparable(desc1, desc2))
+		if (!BTR_types_comparable(matchDesc, valueDesc))
 			return false;
 
-		// if the indexed column is of type int64, we need to inject an
-		// extra cast to deliver the scale value to the BTR level
-
-		if (desc1.dsc_dtype == dtype_int64)
-		{
-			CastNode* cast = FB_NEW_POOL(*tdbb->getDefaultPool()) CastNode(*tdbb->getDefaultPool());
-			cast->source = value;
-			cast->castDesc = desc1;
-			cast->impureOffset = CMP_impure(csb, sizeof(impure_value));
-
-			value = cast;
-
-			if (value2)
-			{
-				cast = FB_NEW_POOL(*tdbb->getDefaultPool()) CastNode(*tdbb->getDefaultPool());
-				cast->source = value2;
-				cast->castDesc = desc1;
-				cast->impureOffset = CMP_impure(csb, sizeof(impure_value));
-
-				value2 = cast;
-			}
-		}
-		else if (desc1.dsc_dtype == dtype_sql_date && desc2.dsc_dtype == dtype_timestamp)
+		if (matchDesc.dsc_dtype == dtype_sql_date &&
+			valueDesc.dsc_dtype == dtype_timestamp)
 		{
 			// for "DATE <op> TIMESTAMP" we need <op> to include the boundary value
 			excludeBound = false;
@@ -1845,11 +1825,11 @@ bool OptimizerRetrieval::matchBoolean(IndexScratch* indexScratch, BoolExprNode*
 
 	const bool isDesc = (indexScratch->idx->idx_flags & idx_descending);
 	int count = 0;
-	IndexScratchSegment** segment = indexScratch->segments.begin();
+	IndexScratchSegment** const segment = indexScratch->segments.begin();
 
 	for (int i = 0; i < indexScratch->idx->idx_count; i++)
 	{
-		FieldNode* fieldNode = match->as<FieldNode>();
+		FieldNode* const fieldNode = match->as<FieldNode>();
 
 		if (!(indexScratch->idx->idx_flags & idx_expressn))
 		{
@@ -1904,10 +1884,7 @@ bool OptimizerRetrieval::matchBoolean(IndexScratch* indexScratch, BoolExprNode*
 					// ASF: Make "NOT boolean" work with indices.
 					case blr_neq:
 					{
-						dsc desc;
-						value->getDesc(tdbb, csb, &desc);
-
-						if (desc.dsc_dtype != dtype_boolean)
+						if (valueDesc.dsc_dtype != dtype_boolean)
 							return false;
 
 						// Let's make a compare with FALSE so we invert the value.
@@ -2046,11 +2023,58 @@ bool OptimizerRetrieval::matchBoolean(IndexScratch* indexScratch, BoolExprNode*
 				// If this is the first segment, then this index is a candidate.
 				indexScratch->candidate = true;
 			}
-
 		}
 	}
 
-	return count >= 1;
+	if (!count)
+		return false;
+
+	// If we have a match and the indexed column is of type int64, we need
+	// to inject an extra cast to deliver the scale value to the BTR level
+
+	if (value && matchDesc.dsc_dtype == dtype_int64)
+	{
+		CastNode *cast1 = NULL, *cast2 = NULL;
+
+		for (int i = 0; i < indexScratch->idx->idx_count; i++)
+		{
+			if (segment[i]->lowerValue == value || segment[i]->upperValue == value)
+			{
+				if (!cast1)
+				{
+					cast1 = FB_NEW_POOL(*tdbb->getDefaultPool()) CastNode(*tdbb->getDefaultPool());
+					cast1->source = value;
+					cast1->castDesc = matchDesc;
+					cast1->impureOffset = CMP_impure(csb, sizeof(impure_value));
+				}
+
+				if (segment[i]->lowerValue == value)
+					segment[i]->lowerValue = cast1;
+
+				if (segment[i]->upperValue == value)
+					segment[i]->upperValue = cast1;
+			}
+
+			if (value2 && (segment[i]->lowerValue == value2 || segment[i]->upperValue == value2))
+			{
+				if (!cast2)
+				{
+					cast2 = FB_NEW_POOL(*tdbb->getDefaultPool()) CastNode(*tdbb->getDefaultPool());
+					cast2->source = value2;
+					cast2->castDesc = matchDesc;
+					cast2->impureOffset = CMP_impure(csb, sizeof(impure_value));
+				}
+
+				if (segment[i]->lowerValue == value2)
+					segment[i]->lowerValue = cast2;
+
+				if (segment[i]->upperValue == value2)
+					segment[i]->upperValue = cast2;
+			}
+		}
+	}
+
+	return true;
 }
 
 InversionCandidate* OptimizerRetrieval::matchDbKey(BoolExprNode* boolean) const
@@ -2466,13 +2490,13 @@ bool OptimizerRetrieval::validateStarts(IndexScratch* indexScratch, ComparativeB
 		// we use starting with against it? Is that allowed?
 		fb_assert(indexScratch->idx->idx_expression != NULL);
 
-		if (!(checkExpressionIndex(indexScratch->idx, field, stream) ||
+		if (!(checkExpressionIndex(csb, indexScratch->idx, field, stream) ||
 			(value && !value->computable(csb, stream, false))))
 		{
 			// AB: Can we swap de left and right sides by a starting with?
 			// X STARTING WITH 'a' that is never the same as 'a' STARTING WITH X
 			if (value &&
-				checkExpressionIndex(indexScratch->idx, value, stream) &&
+				checkExpressionIndex(csb, indexScratch->idx, value, stream) &&
 				field->computable(csb, stream, false))
 			{
 				field = value;
