@@ -180,7 +180,7 @@ namespace
 				*tdbb->getDefaultPool());
 
 			DmlNode* relationNode = PAR_parse_node(tdbb, csb);
-			if (relationNode->kind != DmlNode::KIND_REC_SOURCE)
+			if (relationNode->getKind() != DmlNode::KIND_REC_SOURCE)
 				PAR_syntax_error(csb, "TABLE");
 
 			RelationSourceNode* relationSource = static_cast<RelationSourceNode*>(relationNode);
@@ -654,6 +654,27 @@ CompoundStmtNode* PAR_make_list(thread_db* tdbb, StmtNodeStack& stack)
 }
 
 
+ULONG PAR_marks(Jrd::CompilerScratch* csb)
+{
+	if (csb->csb_blr_reader.getByte() != blr_marks)
+		PAR_syntax_error(csb, "blr_marks");
+
+	switch (csb->csb_blr_reader.getByte())
+	{
+	case 1:
+		return csb->csb_blr_reader.getByte();
+
+	case 2:
+		return csb->csb_blr_reader.getWord();
+
+	case 4:
+		return csb->csb_blr_reader.getLong();
+	}
+	PAR_syntax_error(csb, "valid length for blr_marks value (1, 2, or 4)");
+	return 0;
+}
+
+
 CompilerScratch* PAR_parse(thread_db* tdbb, const UCHAR* blr, ULONG blr_length,
 	bool internal_flag, ULONG dbginfo_length, const UCHAR* dbginfo)
 {
@@ -1550,7 +1571,7 @@ BoolExprNode* PAR_parse_boolean(thread_db* tdbb, CompilerScratch* csb)
 {
 	DmlNode* node = PAR_parse_node(tdbb, csb);
 
-	if (node->kind != DmlNode::KIND_BOOLEAN)
+	if (node->getKind() != DmlNode::KIND_BOOLEAN)
 		PAR_syntax_error(csb, "boolean");
 
 	return static_cast<BoolExprNode*>(node);
@@ -1561,7 +1582,7 @@ ValueExprNode* PAR_parse_value(thread_db* tdbb, CompilerScratch* csb)
 {
 	DmlNode* node = PAR_parse_node(tdbb, csb);
 
-	if (node->kind != DmlNode::KIND_VALUE)
+	if (node->getKind() != DmlNode::KIND_VALUE)
 		PAR_syntax_error(csb, "value");
 
 	return static_cast<ValueExprNode*>(node);
@@ -1572,7 +1593,7 @@ StmtNode* PAR_parse_stmt(thread_db* tdbb, CompilerScratch* csb)
 {
 	DmlNode* node = PAR_parse_node(tdbb, csb);
 
-	if (node->kind != DmlNode::KIND_STATEMENT)
+	if (node->getKind() != DmlNode::KIND_STATEMENT)
 		PAR_syntax_error(csb, "statement");
 
 	return static_cast<StmtNode*>(node);
@@ -1629,7 +1650,7 @@ DmlNode* PAR_parse_node(thread_db* tdbb, CompilerScratch* csb)
 	DmlNode* node = blr_parsers[blr_operator](tdbb, *tdbb->getDefaultPool(), csb, blr_operator);
 	FB_SIZE_T pos = 0;
 
-	if (node->kind == DmlNode::KIND_STATEMENT && csb->csb_dbg_info->blrToSrc.find(blr_offset, pos))
+	if (node->getKind() == DmlNode::KIND_STATEMENT && csb->csb_dbg_info->blrToSrc.find(blr_offset, pos))
 	{
 		MapBlrToSrcItem& i = csb->csb_dbg_info->blrToSrc[pos];
 		StmtNode* stmt = static_cast<StmtNode*>(node);
