@@ -112,7 +112,7 @@ void IndexTableScan::close(thread_db* tdbb) const
 bool IndexTableScan::getRecord(thread_db* tdbb) const
 {
 	if (--tdbb->tdbb_quantum < 0)
-		JRD_reschedule(tdbb, 0, true);
+		JRD_reschedule(tdbb, true);
 
 	jrd_req* const request = tdbb->getRequest();
 	record_param* const rpb = &request->req_rpb[m_stream];
