@@ -788,7 +788,7 @@ Service::Service(const TEXT* service_name, USHORT spb_length, const UCHAR* spb_d
 
 					string trusted_role;
 					mapUser(svc_username, trusted_role, NULL, &svc_auth_block, svc_auth_block,
-						"services manager", NULL, config->getSecurityDatabase(), svc_crypt_callback, NULL);
+						"services manager", NULL, config->getSecurityDatabase(), svc_crypt_callback, NULL, true);
 					trusted_role.upper();
 					svc_trusted_role = trusted_role == ADMIN_ROLE;
 				}
@@ -915,8 +915,11 @@ void Service::detach()
 	// save it cause after call to finish() we can't access class members any more
 	const bool localDoShutdown = svc_do_shutdown;
 
-	TraceServiceImpl service(this);
-	svc_trace_manager->event_service_detach(&service, ITracePlugin::RESULT_SUCCESS);
+	if (svc_trace_manager->needs(ITraceFactory::TRACE_EVENT_SERVICE_DETACH))
+	{
+		TraceServiceImpl service(this);
+		svc_trace_manager->event_service_detach(&service, ITracePlugin::RESULT_SUCCESS);
+	}
 
 	// Mark service as detached.
 	finish(SVC_detached);
@@ -1361,11 +1364,13 @@ ISC_STATUS Service::query2(thread_db* /*tdbb*/,
 		case isc_info_svc_user_dbpath:
 			if (svc_user_flag & SVC_user_dba)
 			{
-				// The path to the user security database (security2.fdb)
-				const RefPtr<const Config> defConf(Config::getDefaultConfig());
-				const char* secDb = defConf->getSecurityDatabase();
+				// The path to the user security database
+				PathName secDb;
+				RefPtr<const Config> config;
+				expandDatabaseName(svc_expected_db, secDb, &config);
+				expandDatabaseName(config->getSecurityDatabase(), secDb, NULL);
 
-				if (!(info = INF_put_item(item, static_cast<USHORT>(strlen(secDb)), secDb, info, end)))
+				if (!(info = INF_put_item(item, static_cast<USHORT>(secDb.length()), secDb.c_str(), info, end)))
 				{
 					return 0;
 				}
@@ -1813,11 +1818,13 @@ void Service::query(USHORT			send_item_length,
 		case isc_info_svc_user_dbpath:
             if (svc_user_flag & SVC_user_dba)
             {
-				// The path to the user security database (security2.fdb)
-				const RefPtr<const Config> defConf(Config::getDefaultConfig());
-				const char* secDb = defConf->getSecurityDatabase();
+				// The path to the user security database
+				PathName secDb;
+				RefPtr<const Config> config;
+				expandDatabaseName(svc_expected_db, secDb, &config);
+				expandDatabaseName(config->getSecurityDatabase(), secDb, NULL);
 
-				if (!(info = INF_put_item(item, static_cast<USHORT>(strlen(secDb)), secDb, info, end)))
+				if (!(info = INF_put_item(item, static_cast<USHORT>(secDb.length()), secDb.c_str(), info, end)))
 				{
 					return;
 				}
@@ -2856,6 +2863,30 @@ bool Service::process_switches(ClumpletReader& spb, string& switches)
 				{
 					string s;
 					spb.getString(s);
+
+					bool inStr = false;
+					for (FB_SIZE_T i = 0; i < s.length(); ++i)
+					{
+						if (s[i] == SVC_TRMNTR)
+						{
+							s.erase(i, 1);
+							if (inStr)
+							{
+								if (i < s.length() && s[i] != SVC_TRMNTR)
+								{
+									inStr = false;
+									continue;
+								}
+							}
+							else
+							{
+								inStr = true;
+								continue;
+							}
+						}
+						++i;
+					}
+
 					switches += s;
 					switches += ' ';
 				}
