@@ -292,7 +292,7 @@ public:
 #ifdef WIRE_COMPRESS_SUPPORT
 		if (slct_zport)
 		{
-			if ((slct_zport->port_flags & PORT_z_data) &&
+			if (slct_zport->port_z_data &&
 				(slct_zport->port_state != rem_port::DISCONNECTED))
 			{
 				port = slct_zport;
@@ -319,7 +319,7 @@ public:
 			return SEL_NO_DATA;
 
 #ifdef WIRE_COMPRESS_SUPPORT
-		if (slct_port->port_flags & PORT_z_data)
+		if (slct_port->port_z_data)
 			return SEL_READY;
 #endif
 
@@ -337,7 +337,7 @@ public:
 	HandleState ok(const rem_port* port)
 	{
 #ifdef WIRE_COMPRESS_SUPPORT
-		if (port->port_flags & PORT_z_data)
+		if (port->port_z_data)
 			return SEL_READY;
 #endif
 		SOCKET n = port->port_handle;
@@ -543,9 +543,7 @@ static SocketsArray* forkSockets;
 static void		get_peer_info(rem_port*);
 
 static void		inet_gen_error(bool, rem_port*, const Arg::StatusVector& v);
-static bool_t	inet_getbytes(XDR*, SCHAR *, unsigned);
 static void		inet_error(bool, rem_port*, const TEXT*, ISC_STATUS, int);
-static bool_t	inet_putbytes(XDR*, const SCHAR*, unsigned);
 static bool		inet_read(XDR*);
 static rem_port*		inet_try_connect(	PACKET*,
 									Rdb*,
@@ -574,17 +572,17 @@ static bool		select_wait(rem_port*, Select*);
 static int		send_full(rem_port*, PACKET *);
 static int		send_partial(rem_port*, PACKET *);
 
-static int		xdrinet_create(XDR*, rem_port*, UCHAR *, USHORT, enum xdr_op);
+static XDR*		xdrinet_create(rem_port*, UCHAR *, USHORT, enum xdr_op);
 static bool		setNoNagleOption(rem_port*);
 static bool		setFastLoopbackOption(rem_port*, SOCKET s = INVALID_SOCKET);
 static FPTR_INT	tryStopMainThread = 0;
 
 
 
-static XDR::xdr_ops inet_ops =
+struct InetXdr : public XDR
 {
-	inet_getbytes,
-	inet_putbytes
+	virtual bool_t x_getbytes(SCHAR *, unsigned);		// get some bytes from "
+	virtual bool_t x_putbytes(const SCHAR*, unsigned);	// put some bytes to "
 };
 
 
@@ -1459,11 +1457,11 @@ static rem_port* alloc_port(rem_port* const parent, const USHORT flags)
 	port->port_async_receive = inet_async_receive;
 	port->port_flags |= flags;
 
-	xdrinet_create(&port->port_send, port,
+	port->port_send = xdrinet_create(port,
 		&port->port_buffer[REM_SEND_OFFSET(INET_remote_buffer)],
 		(USHORT) INET_remote_buffer, XDR_ENCODE);
 
-	xdrinet_create(&port->port_receive, port,
+	port->port_receive = xdrinet_create(port,
 		&port->port_buffer[REM_RECV_OFFSET(INET_remote_buffer)], 0, XDR_DECODE);
 
 	if (parent && !(parent->port_server_flags & SRVR_thread_per_port))
@@ -2052,18 +2050,14 @@ static rem_port* receive( rem_port* main_port, PACKET * packet)
 	// this level rather than try to catch them in all places where
 	// this routine is called
 
-#ifdef DEV_BUILD
-	main_port->port_receive.x_client = !(main_port->port_flags & PORT_server);
-#endif
 	do {
-		if (!xdr_protocol(&main_port->port_receive, packet))
+		if (!xdr_protocol(main_port->port_receive, packet))
 		{
-			packet->p_operation = main_port->port_flags & PORT_partial_data ? op_partial : op_exit;
-			main_port->port_flags &= ~PORT_partial_data;
-
-			if (packet->p_operation == op_exit) {
+			packet->p_operation = main_port->port_partial_data ? op_partial : op_exit;
+			if (packet->p_operation == op_exit)
 				main_port->port_state = rem_port::BROKEN;
-			}
+
+			main_port->port_partial_data = 0;
 			break;
 		}
 #ifdef DEBUG
@@ -2130,7 +2124,7 @@ static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSH
 					*length = 0;
 				}
 #ifdef WIRE_COMPRESS_SUPPORT
-				if (port->port_flags & PORT_z_data)
+				if (port->port_z_data)
 					INET_select->setZDataPort(port);
 #endif
 				return (*length) ? true : false;
@@ -2158,7 +2152,7 @@ static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSH
 				*length = 0;
 			}
 #ifdef WIRE_COMPRESS_SUPPORT
-			if (port->port_flags & PORT_z_data)
+			if (port->port_z_data)
 				INET_select->setZDataPort(port);
 #endif
 			return (*length) ? true : false;
@@ -2439,10 +2433,7 @@ static int send_full( rem_port* port, PACKET * packet)
  *
  **************************************/
 
-#ifdef DEV_BUILD
-	port->port_send.x_client = !(port->port_flags & PORT_server);
-#endif
-	if (!xdr_protocol(&port->port_send, packet))
+	if (!xdr_protocol(port->port_send, packet))
 		return false;
 
 #ifdef DEBUG
@@ -2458,7 +2449,7 @@ static int send_full( rem_port* port, PACKET * packet)
 	} // end scope
 #endif
 
-	return REMOTE_deflate(&port->port_send, inet_write, packet_send, true);
+	return REMOTE_deflate(port->port_send, inet_write, packet_send, true);
 }
 
 static int send_partial( rem_port* port, PACKET * packet)
@@ -2487,15 +2478,11 @@ static int send_partial( rem_port* port, PACKET * packet)
 	} // end scope
 #endif
 
-#ifdef DEV_BUILD
-	port->port_send.x_client = !(port->port_flags & PORT_server);
-#endif
-
-	return xdr_protocol(&port->port_send, packet);
+	return xdr_protocol(port->port_send, packet);
 }
 
 
-static int xdrinet_create(XDR* xdrs, rem_port* port, UCHAR* buffer, USHORT length, enum xdr_op x_op)
+XDR* xdrinet_create(rem_port* port, UCHAR* buffer, USHORT length, enum xdr_op x_op)
 {
 /**************************************
  *
@@ -2508,13 +2495,12 @@ static int xdrinet_create(XDR* xdrs, rem_port* port, UCHAR* buffer, USHORT lengt
  *
  **************************************/
 
-	xdrs->x_public = (caddr_t) port;
-	xdrs->x_base = xdrs->x_private = reinterpret_cast<SCHAR*>(buffer);
-	xdrs->x_handy = length;
-	xdrs->x_ops = (xdr_t::xdr_ops*) &inet_ops;
-	xdrs->x_op = x_op;
+	XDR* xdrs = FB_NEW InetXdr;
 
-	return true;
+	xdrs->x_public = port;
+	xdrs->create(reinterpret_cast<SCHAR*>(buffer), length, x_op);
+
+	return xdrs;
 }
 
 #ifdef HAVE_SETITIMER
@@ -2595,7 +2581,7 @@ static void inet_gen_error(bool releasePort, rem_port* port, const Arg::StatusVe
 }
 
 
-static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, unsigned bytecount)
+bool_t InetXdr::x_getbytes(SCHAR* buff, unsigned bytecount)
 {
 /**************************************
  *
@@ -2607,32 +2593,31 @@ static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, unsigned bytecount)
  *	Get a bunch of bytes from a memory stream if it fits.
  *
  **************************************/
-	const rem_port* port = (rem_port*) xdrs->x_public;
-	if (port->port_flags & PORT_server)
-		return REMOTE_getbytes(xdrs, buff, bytecount);
+	if (x_public->port_flags & PORT_server)
+		return REMOTE_getbytes(this, buff, bytecount);
 
 	// Use memcpy to optimize bulk transfers.
 
 	while (bytecount > sizeof(ISC_QUAD))
 	{
-		if (xdrs->x_handy >= bytecount)
+		if (x_handy >= bytecount)
 		{
-			memcpy(buff, xdrs->x_private, bytecount);
-			xdrs->x_private += bytecount;
-			xdrs->x_handy -= bytecount;
+			memcpy(buff, x_private, bytecount);
+			x_private += bytecount;
+			x_handy -= bytecount;
 			return TRUE;
 		}
 
-		if (xdrs->x_handy > 0)
+		if (x_handy > 0)
 		{
-			memcpy(buff, xdrs->x_private, xdrs->x_handy);
-			xdrs->x_private += xdrs->x_handy;
-			buff += xdrs->x_handy;
-			bytecount -= xdrs->x_handy;
-			xdrs->x_handy = 0;
+			memcpy(buff, x_private, x_handy);
+			x_private += x_handy;
+			buff += x_handy;
+			bytecount -= x_handy;
+			x_handy = 0;
 		}
 
-		if (!inet_read(xdrs))
+		if (!inet_read(this))
 			return FALSE;
 	}
 
@@ -2642,21 +2627,21 @@ static bool_t inet_getbytes( XDR* xdrs, SCHAR* buff, unsigned bytecount)
 	if (!bytecount)
 		return TRUE;
 
-	if (xdrs->x_handy >= bytecount)
+	if (x_handy >= bytecount)
 	{
-		xdrs->x_handy -= bytecount;
+		x_handy -= bytecount;
 		while (bytecount--)
-			*buff++ = *xdrs->x_private++;
+			*buff++ = *x_private++;
 
 		return TRUE;
 	}
 
 	while (bytecount--)
 	{
-		if (xdrs->x_handy == 0 && !inet_read(xdrs))
+		if (x_handy == 0 && !inet_read(this))
 			return FALSE;
-		*buff++ = *xdrs->x_private++;
-		--xdrs->x_handy;
+		*buff++ = *x_private++;
+		--x_handy;
 	}
 
 	return TRUE;
@@ -2725,7 +2710,7 @@ static void inet_error(bool releasePort, rem_port* port, const TEXT* function, I
 	}
 }
 
-static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, unsigned bytecount)
+bool_t InetXdr::x_putbytes(const SCHAR* buff, unsigned bytecount)
 {
 /**************************************
  *
@@ -2742,24 +2727,24 @@ static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, unsigned bytecount)
 
 	while (bytecount > sizeof(ISC_QUAD))
 	{
-		if (xdrs->x_handy >= bytecount)
+		if (x_handy >= bytecount)
 		{
-			memcpy(xdrs->x_private, buff, bytecount);
-			xdrs->x_private += bytecount;
-			xdrs->x_handy -= bytecount;
+			memcpy(x_private, buff, bytecount);
+			x_private += bytecount;
+			x_handy -= bytecount;
 			return TRUE;
 		}
 
-		if (xdrs->x_handy > 0)
+		if (x_handy > 0)
 		{
-			memcpy(xdrs->x_private, buff, xdrs->x_handy);
-			xdrs->x_private += xdrs->x_handy;
-			buff += xdrs->x_handy;
-			bytecount -= xdrs->x_handy;
-			xdrs->x_handy = 0;
+			memcpy(x_private, buff, x_handy);
+			x_private += x_handy;
+			buff += x_handy;
+			bytecount -= x_handy;
+			x_handy = 0;
 		}
 
-		if (!REMOTE_deflate(xdrs, inet_write, packet_send, false))
+		if (!REMOTE_deflate(this, inet_write, packet_send, false))
 		{
 			return FALSE;
 		}
@@ -2771,21 +2756,21 @@ static bool_t inet_putbytes( XDR* xdrs, const SCHAR* buff, unsigned bytecount)
 	if (!bytecount)
 		return TRUE;
 
-	if (xdrs->x_handy >= bytecount)
+	if (x_handy >= bytecount)
 	{
-		xdrs->x_handy -= bytecount;
+		x_handy -= bytecount;
 		while (bytecount--)
-			*xdrs->x_private++ = *buff++;
+			*x_private++ = *buff++;
 
 		return TRUE;
 	}
 
 	while (bytecount--)
 	{
-		if (xdrs->x_handy == 0 && !REMOTE_deflate(xdrs, inet_write, packet_send, false))
+		if (x_handy == 0 && !REMOTE_deflate(this, inet_write, packet_send, false))
 			return FALSE;
-		--xdrs->x_handy;
-		*xdrs->x_private++ = *buff++;
+		--x_handy;
+		*x_private++ = *buff++;
 	}
 
 	return TRUE;
@@ -2806,7 +2791,7 @@ static bool inet_read( XDR* xdrs)
  *	message sent will handle this.
  *
  **************************************/
-	rem_port* port = (rem_port*) xdrs->x_public;
+	rem_port* port = xdrs->x_public;
 	char* p = xdrs->x_base;
 	const char* const end = p + INET_remote_buffer;
 
@@ -2819,7 +2804,7 @@ static bool inet_read( XDR* xdrs)
 	}
 
 	SSHORT length = end - p;
-	port->port_flags &= ~PORT_z_data;
+	port->port_z_data = 0;
 	if (!REMOTE_inflate(port, packet_receive2, (UCHAR*)p, length, &length))
 		return false;
 	p += length;
@@ -2929,7 +2914,7 @@ static bool inet_write(XDR* xdrs)
  **************************************/
 	// Encode the data portion of the packet
 
-	rem_port* port = (rem_port*) xdrs->x_public;
+	rem_port* port = xdrs->x_public;
 	const char* p = xdrs->x_base;
 	USHORT length = xdrs->x_private - p;
 
