@@ -611,7 +611,7 @@ inline void Rsr::releaseException()
 	rsr_status = NULL;
 }
 
-#include "../common/xdr.h"
+#include "../remote/remot_proto.h"
 
 
 // Generalized port definition.
@@ -891,13 +891,13 @@ const USHORT PORT_async			= 0x0002;	// Port is asynchronous channel for events
 const USHORT PORT_no_oob		= 0x0004;	// Don't send out of band data
 const USHORT PORT_disconnect	= 0x0008;	// Disconnect is in progress
 const USHORT PORT_dummy_pckt_set= 0x0010;	// A dummy packet interval is set
-const USHORT PORT_partial_data	= 0x0020;	// Physical packet doesn't contain all API packet
+//const USHORT PORT_partial_data	= 0x0020;	// Physical packet doesn't contain all API packet
 const USHORT PORT_lazy			= 0x0040;	// Deferred operations are allowed
 const USHORT PORT_server		= 0x0080;	// Server (not client) port
 const USHORT PORT_detached		= 0x0100;	// op_detach, op_drop_database or op_service_detach was processed
 const USHORT PORT_rdb_shutdown	= 0x0200;	// Database is shut down
 const USHORT PORT_connecting	= 0x0400;	// Aux connection waits for a channel to be activated by client
-const USHORT PORT_z_data		= 0x0800;	// Zlib incoming buffer has data left after decompression
+//const USHORT PORT_z_data		= 0x0800;	// Zlib incoming buffer has data left after decompression
 const USHORT PORT_compressed	= 0x1000;	// Compress outgoing stream (does not affect incoming)
 const USHORT PORT_released		= 0x2000;	// release(), complementary to the first addRef() in constructor, was called 
 
@@ -955,6 +955,9 @@ struct rem_port : public Firebird::GlobalStorage, public Firebird::RefCounted
 	USHORT			port_protocol;		// protocol version number
 	USHORT			port_buff_size;		// port buffer size
 	USHORT			port_flags;			// Misc flags
+	Firebird::AtomicCounter
+					port_partial_data,	// Physical packet doesn't contain all API packet
+					port_z_data;		// Zlib incoming buffer has data left after decompression
 	SLONG			port_connect_timeout;   // Connection timeout value
 	SLONG			port_dummy_packet_interval; // keep alive dummy packet interval
 	SLONG			port_dummy_timeout;	// time remaining until keepalive packet
@@ -963,14 +966,14 @@ struct rem_port : public Firebird::GlobalStorage, public Firebird::RefCounted
 	struct linger	port_linger;		// linger value as defined by SO_LINGER
 	Rdb*			port_context;
 	Thread::Handle	port_events_thread;	// handle of thread, handling incoming events
-	ThreadId		port_events_threadId;
+	Thread			port_events_threadId;
 	RemotePortGuard* port_thread_guard;	// will close port_events_thread in safe way
 #ifdef WIN_NT
 	HANDLE			port_pipe;			// port pipe handle
 	HANDLE			port_event;			// event associated with a port
 #endif
-	XDR				port_receive;
-	XDR				port_send;
+	Firebird::AutoPtr<XDR>	port_receive;
+	Firebird::AutoPtr<XDR>	port_send;
 #ifdef DEBUG_XDR_MEMORY
 	r e m _ v e c*	port_packet_vector;		// Vector of send/receive packets
 #endif
@@ -1031,9 +1034,10 @@ public:
 		port_type(t), port_state(PENDING), port_clients(0), port_next(0),
 		port_parent(0), port_async(0), port_async_receive(0),
 		port_server(0), port_server_flags(0), port_protocol(0), port_buff_size(rpt / 2),
-		port_flags(0), port_connect_timeout(0), port_dummy_packet_interval(0),
+		port_flags(0), port_partial_data(0), port_z_data(0),
+		port_connect_timeout(0), port_dummy_packet_interval(0),
 		port_dummy_timeout(0), port_handle(INVALID_SOCKET), port_channel(INVALID_SOCKET), port_context(0),
-		port_events_thread(0), port_events_threadId(0), port_thread_guard(0),
+		port_events_thread(0), port_thread_guard(0),
 #ifdef WIN_NT
 		port_pipe(INVALID_HANDLE_VALUE), port_event(INVALID_HANDLE_VALUE),
 #endif
@@ -1144,7 +1148,7 @@ public:
 
 	void releaseObject(OBJCT id)
 	{
-		if (id != INVALID_OBJECT)
+		if (id != INVALID_OBJECT && id <= MAX_OBJCT_HANDLES)
 		{
 			port_objects[id].release();
 		}
@@ -1170,7 +1174,7 @@ public:
 	bool haveRecvData()
 	{
 		Firebird::RefMutexGuard queGuard(*port_que_sync, FB_FUNCTION);
-		return ((port_receive.x_handy > 0) || (port_qoffset < port_queue.getCount()));
+		return ((port_receive->x_handy > 0) || (port_qoffset < port_queue.getCount()));
 	}
 
 	void clearRecvQue()
@@ -1178,20 +1182,20 @@ public:
 		Firebird::RefMutexGuard queGuard(*port_que_sync, FB_FUNCTION);
 		port_queue.clear();
 		port_qoffset = 0;
-		port_receive.x_private = port_receive.x_base;
+		port_receive->x_private = port_receive->x_base;
 	}
 
 	class RecvQueState
 	{
 	public:
-		int save_handy;
+		unsigned save_handy;
 		FB_SIZE_T save_private;
 		FB_SIZE_T save_qoffset;
 
 		RecvQueState(const rem_port* port)
 		{
-			save_handy = port->port_receive.x_handy;
-			save_private = port->port_receive.x_private - port->port_receive.x_base;
+			save_handy = port->port_receive->x_handy;
+			save_private = port->port_receive->x_private - port->port_receive->x_base;
 			save_qoffset = port->port_qoffset;
 		}
 	};
@@ -1206,11 +1210,11 @@ public:
 		if (rs.save_qoffset > 0 && (rs.save_qoffset != port_qoffset))
 		{
 			Firebird::Array<char>& q = port_queue[rs.save_qoffset - 1];
-			memcpy(port_receive.x_base, q.begin(), q.getCount());
+			memcpy(port_receive->x_base, q.begin(), q.getCount());
 		}
 		port_qoffset = rs.save_qoffset;
-		port_receive.x_private = port_receive.x_base + rs.save_private;
-		port_receive.x_handy = rs.save_handy;
+		port_receive->x_private = port_receive->x_base + rs.save_private;
+		port_receive->x_handy = rs.save_handy;
 	}
 
 	// TMN: The following member functions are conceptually private
@@ -1258,7 +1262,7 @@ public:
 
 	Firebird::string getRemoteId() const;
 	void auxAcceptError(PACKET* packet);
-	void addServerKeys(CSTRING* str);
+	void addServerKeys(const CSTRING* str);
 	bool tryNewKey(InternalCryptKey* cryptKey);
 	void checkResponse(Firebird::IStatus* warning, PACKET* packet, bool checkKeys = false);
 
