@@ -522,7 +522,8 @@ namespace
 		int release();
 
 	private:
-		~ConfiguredPlugin();
+		~ConfiguredPlugin() {}
+		void destroy();
 
 		RefPtr<PluginModule> module;
 		unsigned int regPlugin;
@@ -685,10 +686,15 @@ namespace
 
 	GlobalPtr<PluginsMap> plugins;
 
-	ConfiguredPlugin::~ConfiguredPlugin()
+	void ConfiguredPlugin::destroy()
 	{
-		MutexLockGuard g(plugins->mutex, FB_FUNCTION);
+		// plugins->mutex must be entered by current thread
 
+#ifdef DEV_BUILD
+		if (!plugins->mutex.tryEnter(FB_FUNCTION))
+			fb_assert(false);
+		plugins->mutex.leave();
+#endif
 		if (!destroyingPluginsMap)
 		{
 			plugins->remove(MapKey(module->getPlugin(regPlugin).type, plugName));
@@ -733,6 +739,16 @@ namespace
 
 		if (x == 0)
 		{
+			{ // plugins->mutex scope
+				MutexLockGuard g(plugins->mutex, FB_FUNCTION);
+				if (refCounter != 0)
+					return 1;
+
+				destroy();
+			}
+
+			// Must run out of mutex scope to avoid deadlock with PluginManager::threadDetach()
+			// called when module is unloaded by dtor
 			delete this;
 			return 0;
 		}
@@ -1080,7 +1096,7 @@ void PluginManager::unregisterModule(IPluginModule* cleanup)
 		Firebird::dDllUnloadTID = GetCurrentThreadId();
 #endif
 
-	fb_shutdown(5000, fb_shutrsn_exit_called);
+	fb_shutdown(10000, fb_shutrsn_exit_called);
 }
 
 IPluginSet* PluginManager::getPlugins(CheckStatusWrapper* status, unsigned int interfaceType,
@@ -1191,17 +1207,19 @@ void PluginManager::threadDetach()
 
 namespace {
 
-class DirCache
+class DataCache
 {
 public:
-	explicit DirCache(MemoryPool &p)
-		: cache(p)
+	explicit DataCache(MemoryPool &p)
+		: cache(p), db(p)
 	{
 		cache.resize(IConfigManager::DIR_COUNT);
 		for (unsigned i = 0; i < IConfigManager::DIR_COUNT; ++i)
 		{
 			cache[i] = fb_utils::getPrefix(i, "");
 		}
+
+		db = fb_utils::getPrefix(IConfigManager::DIR_SECDB, "security3.fdb");
 	}
 
 	const char* getDir(unsigned code)
@@ -1210,11 +1228,17 @@ public:
 		return cache[code].c_str();
 	}
 
+	const char* getDb()
+	{
+		return db.c_str();
+	}
+
 private:
 	ObjectsArray<PathName> cache;
+	PathName db;
 };
 
-InitInstance<DirCache> dirCache;
+InitInstance<DataCache> dataCache;
 
 }	// anonymous namespace
 
@@ -1236,7 +1260,7 @@ public:
 	{
 		try
 		{
-			return dirCache().getDir(code);
+			return dataCache().getDir(code);
 		}
 		catch (const Exception&)
 		{
@@ -1297,6 +1321,12 @@ public:
 	{
 		return rootDetector().getRootDirectory();
 	}
+
+	const char* getDefaultSecurityDb()
+	{
+		return dataCache().getDb();
+	}
+
 };
 
 static ConfigManager configManager;
