@@ -586,7 +586,10 @@ YAttachment* UtilInterface::executeCreateDatabase(
 		if (stmtIsCreateDb)
 			*stmtIsCreateDb = FB_FALSE;
 
-		if (!PREPARSE_execute(status, &att, stmtLength, creatDBstatement, &stmtEaten, dialect))
+		string statement(creatDBstatement,
+			(stmtLength == 0 && creatDBstatement ? strlen(creatDBstatement) : stmtLength));
+
+		if (!PREPARSE_execute(status, &att, statement, &stmtEaten, dialect))
 			return NULL;
 
 		if (stmtIsCreateDb)
@@ -610,7 +613,7 @@ YAttachment* UtilInterface::executeCreateDatabase(
 
 		if (!stmtEaten)
 		{
-			att->execute(status, crdbTrans, stmtLength, creatDBstatement, dialect, NULL, NULL, NULL, NULL);
+			att->execute(status, crdbTrans, statement.length(), statement.c_str(), dialect, NULL, NULL, NULL, NULL);
 			if (status->getState() & Firebird::IStatus::STATE_ERRORS)
 			{
 				crdbTrans->rollback(&tempCheckStatusWrapper);
